const Realm = require('realm');

// 创建数据模型，并且在 properties 中创建属性
const MateData = {
    name: 'MateData',
    properties: {
        make:  'string',
        model: 'string',
        miles: {type: 'int', default: 0},
    }
};
const PersonSchema = {
    name: 'Person',
    properties: {
        name:     'string',
        birthday: 'date',
        cars:     'Car[]',
        picture:  'data?' // optional property
    }
};

//realm 使用的特别之处，把创建的数据模型整合到 schema  之中,通过 open 方法打开一条 Realm
Realm.open({schema: [MateData]})
    .then(realm => {
        // 只能在 write 方法中操作数据，查询则不用
        realm.write(() => {
            //create 创建数据的方法
            const MateData = realm.create('MateData', {
                make: 'Honda',
                model: 'Civic',
                miles: 1000,
            });
            // 更新数据
            MateData.miles += 20;
        });

        // 查询,返回数组
        const MateData = realm.objects('MateData').filtered('miles > 1000');

        //这个时候长度是1
        console.log(MateData.length); // => 1

        // 再次新增另一条数据
        realm.write(() => {
            const MateData = realm.create('MateData', {
                make: 'Ford',
                model: 'Focus',
                miles: 2000,
            });
            MateData.miles += 20;
        });

        // 查询结果更新，变成2
        console.log(MateData.length); // => 2
    })
    .catch(error => {
        console.log(error);
    });
