import Realm from 'realm';

/***表定义区**/
export const BasicTableName = 'basic';
export const CwTableName = 'cw_table';
export const NullTableName = 'nullAction';


const NullSchema = {
    name: NullTableName,
    primaryKey: 'id',
    properties: {
        id: 'int',
        errorName: 'string',
        errorCode: 'string',
        platform: 'string',
        eventName: 'string',
        value: 'string',
        valueName: 'string'
    }
};
const BasicTableSchema = {
    name: BasicTableName,
    primaryKey: 'id',
    properties: {
        id: 'int',
        name: 'string',
    }
};


const CwTableSchema = {
    name: CwTableName,
    primaryKey: 'city_id',
    properties: {
        city_id: 'int',
        city_name: 'string',
    }
};

const instance = new Realm({
    schema: [
        BasicTableSchema,
        CwTableSchema,
        NullSchema,
    ],
    deleteRealmIfMigrationNeeded: true,
    inMemory: false,
});


/***表使用区**/
export function writeToRealm(obj,tabName) {
    return new Promise((resolve, reject) => {
        instance.write(() => {
            instance.create(tabName, obj, true)
            resolve(true)
        })
    })
}


export function queryAllFromRealm(tabName) {
    return new Promise((resolve, reject) => {
        let obj = instance.objects(tabName);
        let objStr = JSON.stringify(obj);
        resolve(JSON.parse(objStr))
    })
}

export function clearAllFromRealm(tabName) {
    return new Promise((resolve, reject) => {
        instance.write(() => {
            let arrays = instance.objects(tabName);
            instance.delete(arrays);
            resolve(true)
        })
    })
}


export function clearRowFromRealm(id,tabName) {
    return new Promise((resolve, reject) => {
        instance.write(() => {
            let arrays = instance.objects(tabName);
            let row = arrays.filtered('id==' + id);
            instance.delete(row);
            resolve(true)
        })
    })
}
